"use strict";

Object.defineProperty(exports, "__esModule", {
  value: true
});
exports.default = void 0;

var path = _interopRequireWildcard(require("path"));

var _core = require("@babel/core");

var _utils = require("./utils");

function _getRequireWildcardCache(nodeInterop) { if (typeof WeakMap !== "function") return null; var cacheBabelInterop = new WeakMap(); var cacheNodeInterop = new WeakMap(); return (_getRequireWildcardCache = function (nodeInterop) { return nodeInterop ? cacheNodeInterop : cacheBabelInterop; })(nodeInterop); }

function _interopRequireWildcard(obj, nodeInterop) { if (!nodeInterop && obj && obj.__esModule) { return obj; } if (obj === null || typeof obj !== "object" && typeof obj !== "function") { return { default: obj }; } var cache = _getRequireWildcardCache(nodeInterop); if (cache && cache.has(obj)) { return cache.get(obj); } var newObj = {}; var hasPropertyDescriptor = Object.defineProperty && Object.getOwnPropertyDescriptor; for (var key in obj) { if (key !== "default" && Object.prototype.hasOwnProperty.call(obj, key)) { var desc = hasPropertyDescriptor ? Object.getOwnPropertyDescriptor(obj, key) : null; if (desc && (desc.get || desc.set)) { Object.defineProperty(newObj, key, desc); } else { newObj[key] = obj[key]; } } } newObj.default = obj; if (cache) { cache.set(obj, newObj); } return newObj; }

const isResoureInSrc = (srcDir, resourecPath) => !path.relative(srcDir, resourecPath).includes('..');

const getImportFileDistPath = (compilerFile, srcDir, importName) => {
  const dir = path.dirname(compilerFile);
  const compilerFileRelativeLoc = path.relative(dir, srcDir);
  const importFileRelativeLoc = path.relative(srcDir, path.join(dir, importName));
  const inSrc = isResoureInSrc(srcDir, path.join(dir, importName));
  const importFileDistPath = path.join(inSrc ? '..' : '../..', compilerFileRelativeLoc, 'styles', importFileRelativeLoc);
  return importFileDistPath;
};

const isStaticFile = file => {
  const tests = [/\.json$/, /\.css$/, /\.less$/, /\.sass$/, /\.scss$/, /\.(jpg|png|jpeg|gif|svg)$/, /\.(jpg|png|jpeg|gif|svg)\?(inline|url)$/];
  return tests.some(regex => regex.test(file));
};

const isStyleFile = file => {
  const tests = [/\.css$/, /\.less$/, /\.sass$/, /\.scss$/];
  return tests.some(regex => regex.test(file));
};

const getReplacePath = (importName, filename, srcDir, importStyle) => {
  if (!filename || !importName) {
    return '';
  }

  if (!isStaticFile(importName)) {
    return '';
  }

  if (!(0, _utils.isProjectFile)(importName)) {
    return '';
  }

  let realFilepath = getImportFileDistPath(filename, srcDir, importName);

  if (isStyleFile(realFilepath) && importStyle === 'compiled-code') {
    realFilepath = realFilepath.replace(/\.(less|sass|scss)$/, '.css');
  }

  return realFilepath;
};

const importPath = () => ({
  name: 'import-path',
  visitor: {
    ImportDeclaration({
      node
    }, {
      opts,
      file
    }) {
      const {
        source
      } = node;
      const {
        appDirectory,
        importStyle = 'source-code'
      } = opts;
      const srcDir = `${appDirectory}/src`;
      const {
        opts: {
          filename
        }
      } = file;
      const importName = source !== null && source !== void 0 && source.value ? source.value : undefined;
      const replaceValue = getReplacePath(importName, filename, srcDir, importStyle);

      if (replaceValue) {
        node.source.value = replaceValue;
      }
    },

    // dynamic import
    CallExpression({
      node
    }, {
      opts,
      file
    }) {
      const {
        appDirectory,
        importStyle = 'source-code'
      } = opts;
      const srcDir = `${appDirectory}/src`;
      const {
        filename
      } = file.opts;
      const {
        callee,
        arguments: args
      } = node;

      if (callee.type === 'Import' || callee.type === 'Identifier' && callee.name === 'require') {
        const firstArg = args[0];

        if (firstArg.value) {
          const importName = firstArg.value;
          const replaceValue = getReplacePath(importName, filename, srcDir, importStyle);

          if (replaceValue) {
            node.arguments = [_core.types.stringLiteral(replaceValue)];
          }
        }
      }
    }

  }
});

var _default = importPath;
exports.default = _default;